#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@User    : frank 
@Time    : 2019/9/22 10:09
@File    : timer.py
@Email  : frank.chang@xinyongfei.cn
"""
import time
from functools import partial, wraps

from loguru import logger

now = now_int = time.time


def fn_timer(fn=None, *, prefix="", interval=0.0):
    """
    计算 fn  的运算时间

    :param fn: 函数, callable 对象
    :param prefix: 给函数一个标识 前缀
    :param interval: float  给定一个阈值 ,超过某一个阈值,就记录时间. 否则不记录,默认是0 ,记录所有的函数的运行时间.
    :return:
    """
    if fn is None:
        return partial(fn_timer, prefix=prefix)

    @wraps(fn)
    def function_timer(*args, **kwargs):
        start = now()
        result = fn(*args, **kwargs)
        t = now() - start
        if t > interval:
            logger.info(f'{prefix}{fn.__name__} total running time {now() - start} seconds')
        return result

    return function_timer


@fn_timer()
def get_token():
    time.sleep(3)
    return 0


@fn_timer(prefix='frank_')
def pickup():
    print("pickup is running.")
    time.sleep(2)
    print("pickup is end.")


if __name__ == '__main__':
    # get_token()
    pickup()


    pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@User    : frank 
@Time    : 2019/9/26 20:46
@File    : func.py
@Email  : frank.chang@xinyongfei.cn


这里检查对象 属性 是否满足 某些条件,进行 特定返回值 .

https://docs.python.org/3/library/inspect.html#inspect.Signature.bind
https://docs.python.org/3/library/inspect.html#inspect.BoundArguments
"""

import functools
import inspect


def checked_arguments(f):
    @functools.wraps(f)
    def wrapper(*a, **k):
        sig = inspect.signature(f).bind(*a, **k)
        sig.apply_defaults()
        d = sig.arguments

        valid_val = check_arguments(d)
        if valid_val:
            return f(*a, **k)
        else:
            return {
                'score': 0,
                'prob': 0
            }

    return wrapper


def check_arguments(d):
    """
    这里是 检查 参数的条件, 这里可以随便写 .
    :param d:
    :return:
    """
    args = d.get('args')
    kw = d.get('kwargs')

    # 这里检查 只要参数存在 就进行计算,否则 不计算
    if not any(args):
        if not any(kw.values()):
            return False

    return True


@checked_arguments
def run(*args, **kwargs):
    """
    模型函数,计算分值用的.
    :param args:
    :param kwargs:
    :return:
    """
    return (args, kwargs)


if __name__ == '__main__':
    print(run(a=1))  # ((), {'a': 1})
    print(run())  # {'score': 0, 'prob': 0}



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@User    : frank 
@Time    : 2019/9/26 20:46
@File    : func.py
@Email  : frank.chang@xinyongfei.cn


这里检查对象 属性 是否满足 某些条件,进行 特定返回值 .

检查 对象中 data 属性 ,是否 满足 某些条件
如果 data 满足 某些条件,可以进行计算 ; 否则不算,直接返回默认值 .


"""

import functools
from loguru import logger

# 默认信用分
DEFAULT_SCORE = -111.0

# 默认概率分
DEFAULT_PROB = -222.0


def _check_arguments(d):
    """

    如果 d 全为空值,None ,则返回False . d.values有一个不为空,None,0 则 返回True .

    """
    copy = d.copy()
    return any(copy.values())


def checked_arguments(score, prob):
    def _checked_arguments(f):
        @functools.wraps(f)
        def wrapper(self, *args, **kwargs):

            valid_val = _check_arguments(self.data)
            if valid_val:
                return f(self, *args, **kwargs)
            else:
                logger.info(
                    f'checked_arguments failed. '
                    f'return default value: DEFAULT_SCORE:{DEFAULT_SCORE}, DEFAULT_PROB:{DEFAULT_PROB} '
                    f'class_name:{self.__class__.__name__}'
                )
                return {
                    score: DEFAULT_SCORE,
                    prob: DEFAULT_PROB
                }

        return wrapper

    return _checked_arguments


class Model:

    def __init__(self, data):
        self.data = data

    @checked_arguments(score='score', prob='prob')
    def calculate(self):
        return {
            "score": 100.0,
            "prob": 100.0,
        }


if __name__ == '__main__':
    data = {'0': None, '1': None, '6': None, '7': None, '8': None, '9': None}

    model = Model(data=data)

    r = model.calculate()

    print(f"r :{r}")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@User    : frank 
@Time    : 2019/9/26 20:46
@File    : func.py
@Email  : frank.chang@xinyongfei.cn

这里检查对象 属性 是否满足 某些条件,进行 特定返回值 .
https://docs.python.org/3/library/inspect.html#inspect.Signature.bind
https://docs.python.org/3/library/inspect.html#inspect.BoundArguments

"""

import functools
import inspect
from loguru import logger
from collections import OrderedDict

default_value = {
    'ret': 0
}


def checked_arguments(*, callback, default):
    """
    :param callback: function  ,
            这个 callable 只接受 一个参数 Orderdict
            this  function 返回  True,or False  如果为 True 则进行 下面的逻辑 fn的逻辑,
            否则不进行 fn 的逻辑,直接返回  规定的默认值default

    :param default: 如果 不能检查 成功 ,返回 default值
    :return:
    """

    def _checked_arguments(f):
        @functools.wraps(f)
        def wrapper(*a, **k):
            sig = inspect.signature(f).bind(*a, **k)
            sig.apply_defaults()
            d = sig.arguments

            valid_val = callback(d)
            if valid_val:

                value = f(*a, **k)
                logger.info(f"return normal_value, value:{value} ")
                return value


            else:
                logger.info(f"return default_value, value:{default} ")
                return default

        return wrapper

    return _checked_arguments


def call_back(params_dict: OrderedDict):
    """
    用户自定义的一个 callback ,这个函数 返回 true or  false


    这里 自定义 a >10 我认为 检查失败
    :param params_dict:
    :return: true or false
    """
    logger.info(f"params_dict: {params_dict}")
    a = params_dict.get('a')

    if a > 10:
        return True
    return False


@checked_arguments(callback=call_back, default=default_value)
def calculate(a, b):
    """
    用户函数
    :param a:
    :param b:
    :return:
    """
    return {"ret": 20}


def case_one():
    print(calculate(2, 20))
    print(calculate(20, 20))


def case_two():
    print(predict(num=100))
    print(predict(num=1))


def predict_callback(data):
    """
    user defined
    :param data: dict
    :return:
    """

    kwargs = data.get('kwargs')
    # print(kwargs)
    num = kwargs.get('num')
    if num > 10:
        return True
    return False


@checked_arguments(callback=predict_callback, default=-100)
def predict(**kwargs):
    """
    用户函数
    :param kargs:
    :return:
    """
    return 100


def check_args(data):
    """
    这里如果判断  参数都小于10 认为可以通过检查
    :param data:
    :return:
    """

    args = data.get('args')
    max_value = max(args)

    if max_value > 10:
        return True

    return False


@checked_arguments(callback=check_args, default=-100)
def fun(*args, **kwargs):
    return 100


if __name__ == '__main__':
    fun(10, 20)
    fun(1, 2)

    pass



# -*- coding: utf-8 -*- 
"""
@User     : Frank
@File     : setup.py
@DateTime : 2019-09-16 11:24 
@Email    : frank.chang@lexisnexis.com
"""
from setuptools import setup,find_packages
import io
import re

with io.open('README.rst', 'r', encoding='utf8') as f:
    long_description = f.read()

with io.open("src/useful_decoration/__init__.py", "rt", encoding="utf8") as f:
    version = re.search(r'__version__ = "(.*?)"', f.read()).group(1)


setup(
    name="useful_decoration",
    license='Apache License 2.0',
    version=version,
    packages=find_packages("src"),
    zip_safe=False,
    include_package_data=True,
    package_dir={"": "src"},
    long_description=long_description,
    url='https://github.com/changyubiao/useful_decoration',
    author='frank',
    author_email='frank.chang@lexisnexis.com',
    description='powerful and useful decorations',

    project_urls={
        "Documentation": "https://useful-decoration.readthedocs.io/en/latest/",
        "Code": "https://github.com/changyubiao/useful_decoration",
    },

    python_requires='>=3.6',
    install_requires=[
        "loguru>=0.3.2",

    ],

)

